`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 27.03.2023 21:03:40
// Design Name: 
// Module Name: FSM_controller
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////


`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 27.03.2023 20:58:37
// Design Name: 
// Module Name: FSM
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////


module FSM_controller(clk, reset, 
           rst_a, shift_a, rst_b, shift_b, carry_sel, // Output control sig. 
           enable_sum, rst_count, inc_count,   // Output control sig.
           counter_is7,      // Input status signal 
           done 
    );

input clk;
input reset;    // Active high. 

output logic rst_a, shift_a, rst_b, shift_b, carry_sel; // Output control sig. 
output logic enable_sum, rst_count, inc_count;   // Output control sig.

input counter_is7;
output done;            // Goes high after completing mp-addition.


reg [1:0] state;    // Onely 2 bits are needed as there could be 4 states.
logic [1:0] nextstate;

always @(posedge clk)
begin
    if(reset)
        state <= 2'd0;
    else
        state <= nextstate;    
end

always @(counter_is7)
begin
    case(state)
    2'd0: begin // Initial reset state.
            rst_a<=1; shift_a<=0; rst_b<=1; shift_b<=0; carry_sel<=0; 
            enable_sum<=0; rst_count<=1; inc_count<=0;
            nextstate<=2'd1;
        end  

    2'd1: begin // Compute a[0]+b[0]+initial_carry. 
                // Enable sum[ ], shift a[ ], shift b[ ]. Increment counter. They update in next cycle
            rst_a<=0; shift_a<=1; rst_b<=0; shift_b<=1; carry_sel<=0; 
            enable_sum<=1; rst_count<=0; inc_count<=1;
            nextstate<=2'd2;
        end  

    2'd2: begin // Compute a[i]+b[i]+Carry_out_reg till the last 8-bit chunk. 
                 // Enable sum[ ], shift a[ ], shift b[ ]. Increment counter. They update in next cycle
            rst_a<=0; shift_a<=1; rst_b<=0; shift_b<=1; carry_sel<=1; 
            enable_sum<=1; rst_count<=0; inc_count<=1;
            if(counter_is7) nextstate<=2'd3;
            else nextstate<=2'd2;
        end  

    2'd3: begin  // End state. 
                 // Disable register updates
            rst_a<=0; shift_a<=0; rst_b<=0; shift_b<=0; carry_sel<=0; 
            enable_sum<=0; rst_count<=0; inc_count<=0;
            nextstate<=2'd3;
        end  

    default: begin  // Same as End state. 
            rst_a<=0; shift_a<=0; rst_b<=0; shift_b<=0; carry_sel<=0; 
            enable_sum<=0; rst_count<=0; inc_count<=0;
            nextstate<=2'd3;
        end         
    endcase
end

assign done = (state==2'd3);
    
endmodule
