`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 27.03.2023 16:25:12
// Design Name: 
// Module Name: dp_mpadder
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////


module dp_mpadder(clk, 
                  a, b, carry_in_initial, // input data variables
                  rst_a, shift_a, rst_b, shift_b, carry_sel, // Input control sig. 
                  enable_sum, rst_count, inc_count,          // Input control sig.
                  counter_is7,      // Output status signal
                  carry_out, sum    // Output data variables       
    );
input clk;
input [63:0] a, b;  // Input data variables for addition
input carry_in_initial;      // Input initial carry_in
input rst_a, shift_a, rst_b, shift_b, carry_sel; // Input control sig. 
input enable_sum, rst_count, inc_count; // Input control sig.

output counter_is7;
output carry_out;
output reg [63:0] sum;    

reg [63:0] a_reg, b_reg;
wire [7:0] add1_a_in, add1_b_in;
wire add1_carry_in;
wire add1_carry_out;
wire [7:0] add1_sum_out;

reg carry_out_reg;

reg [2:0] counter_i;

always @(posedge clk)
begin
    if(rst_a)
        a_reg <= a;
    else if (shift_a)
        a_reg <= {8'd0, a_reg[63:8]};
    else
        a_reg <= a_reg;       
end

always @(posedge clk)
begin
    if(rst_b)
        b_reg <= b;
    else if (shift_b)
        b_reg <= {8'd0, b_reg[63:8]};
    else
        b_reg <= b_reg;       
end


mux2to1_1bit mux1(carry_in_initial, carry_out_reg, carry_sel, add1_carry_in);


assign add1_a_in = a_reg[7:0]; 
assign add1_b_in = b_reg[7:0];

adder8bit   Add1(add1_a_in, add1_b_in, add1_carry_in, 
                 add1_carry_out, add1_sum_out);

always @(posedge clk)
begin    
    if(enable_sum)  // Carry is stored only during sum.
        carry_out_reg <= add1_carry_out; 
    else
        carry_out_reg <= carry_out_reg;
end

/*
always @(posedge clk)
begin
    if(enable_sum)
        sum <= {add1_sum_out, sum[63:8]};
    else
        sum <= sum;
end
*/

always @(posedge clk)
begin

    if(rst_count)
        counter_i <= 3'd0;
    else if(inc_count)
        counter_i <= counter_i + 1;
    else
        counter_i <= counter_i;        


    if(enable_sum)
        sum <= {add1_sum_out, sum[63:8]};
    else
        sum <= sum;


end

assign counter_is7 = (counter_i==3'd7) ? 1'b1 : 1'b0;

assign carry_out = carry_out_reg;

endmodule
